/*(c) JippySnake7 Developmentteam.
 *
 *
 * This code is published unter GNU Public Licence v2.
 *
 *
 */
import java.io.*;
import java.net.*;
class JSLevel {
	private int[][] layerDown;
	private int[][] layerTop;
	private int[][][] layer;
	private boolean[][][] apples;
	private boolean[][][] snake;
	
	private int minApples;
	private int applesTillNew;
	private int applesTillNewGlobal;
	private int shownApples;
	private int numApples; //Apples that have been eaten
	
	private int numLayers;
	private int number;
	private int type;
	
	private boolean timerUsed;
	private int time; //Time is saved in quarterseconds time=4 <=> time=1s
	
	static JSLevel loadLevel(String file) {
		JSLevel l = new JSLevel();
		l.loadNewLevel(file);
		return l;
	}
	
	public void loadNewLevel(String file) {
		try {
			URI texFile = new URI(new URL(Globals.GAMEROOT+file).toString());
			FileReader fileReader = new FileReader(new File(texFile));
			BufferedReader buffReader = new BufferedReader(fileReader);
			String line = buffReader.readLine();
			String[] parts = line.split("::");
			while (!line.equals("@@@")) {
				parts = line.split("::");
				if (parts[0].equals("TYPE")) {
					/* Game Types:
					   0: Classic snake
					   1: "Fressen" you have to eat all aplles in the level
					   2: "Vielfrass" First you have to 1 one apple to get new, then 2, aso...
					   3: "Verwelken" You have to eat apples in a specifik time
					   4: "Blind-snake" You see only a short area around you head
					*/
					type = Integer.parseInt(parts[1]);
					if (type == 3) {
						timerUsed = true;
						time = 360; //90 seconds
					} else {
						timerUsed = false;
						time = 99999; //some value
					}
				}
				else if (parts[0].equals("NUMBER")) {
					number = Integer.parseInt(parts[1]);
				} else if (parts[0].equals("LAYERS")) {
					numLayers = Integer.parseInt(parts[1]);
					layer = new int[numLayers][Globals.SIZE_X][Globals.SIZE_Y];
					apples = new boolean[numLayers][Globals.SIZE_X][Globals.SIZE_Y];
					snake = new boolean[numLayers][Globals.SIZE_X][Globals.SIZE_Y];
				} else if (parts[0].equals("APPLESTOEAT"))
					minApples = Integer.parseInt(parts[1]);
				else if (parts[0].equals("APPLESTILLNEW")) {
					applesTillNew = Integer.parseInt(parts[1]);
					applesTillNewGlobal = Integer.parseInt(parts[1]);
				} else if (parts[0].equals("SHOWNAPPLES"))
					shownApples = Integer.parseInt(parts[1]);
				else if (parts[0].equals("LAYER_DOWN")) {
					layerDown = new int[Globals.SIZE_X][Globals.SIZE_Y];
					for (int y = 0; y < Globals.SIZE_Y; y++) {
						line = buffReader.readLine();
						parts = line.split(" ");
						for (int x = 0; x < Globals.SIZE_X; x++)
							layerDown[x][y] = Integer.parseInt(parts[x]);
					}
				} else if (parts[0].equals("LAYER")) {
					int layerNum = Integer.parseInt(parts[1]);
					for (int y = 0; y < Globals.SIZE_Y; y++) {
						line = buffReader.readLine();
						parts = line.split(" ");
						for (int x = 0; x < Globals.SIZE_X; x++)
							layer[layerNum][x][y] = Integer.parseInt(parts[x]);
					}
				} else if (parts[0].equals("LAYER_TOP")) {
					layerTop = new int[Globals.SIZE_X][Globals.SIZE_Y];
					for (int y = 0; y < Globals.SIZE_Y; y++) {
						line = buffReader.readLine();
						parts = line.split(" ");
						for (int x = 0; x < Globals.SIZE_X; x++)
							layerTop[x][y] = Integer.parseInt(parts[x]);
					}
				} else if (line.equals("")) {
				} else
					System.out.println("Unkown data: "+line);
				line = buffReader.readLine();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		placeApples(shownApples, new Coord(-1,-1,-1));
		numApples = 0;
	}
	
	public void reInit() {
		time = 360;
		if (type == 2) applesTillNewGlobal = 1;
		if (type == 3) timerUsed = true;
		
		applesTillNew = applesTillNewGlobal;
		for (Coord crd : Coord.range(Coord.getZeroCoord(), new Coord(Globals.SIZE_X, Globals.SIZE_Y, numLayers).add(new Coord(-1,-1,-1)))) {
			snake[crd.getZ()][crd.getX()][crd.getY()] = false;
			apples[crd.getZ()][crd.getX()][crd.getY()] = false;
		}
		numApples = 0;
		placeApples(shownApples, new Coord(-1,-1,-1));
	}
	
	public int getType() {
		return type;
	}
	public int getLevelNum() {
		return number;
	}
	public int getNumLayers() {
		return numLayers;
	}
	public int getNumApples() {
		return numApples;
	}
	public int getMinApples() {
		return minApples;
	}
	public int applesTillNewGlobal() {
		return applesTillNewGlobal;
	}
	public int applesTillNew() {
		return applesTillNew;
	}
	public int getNumShownApples() {
		return shownApples;
	}
	public boolean timerUsed() {
		return timerUsed;
	}
	public void setTimerUsed(boolean v) {
		timerUsed = v;
	}
	public void decTime() {
		time--;
	}
	public int remainingTime() {
		return time;
	}
	public Coord getSize() {
		return new Coord(Globals.SIZE_X, Globals.SIZE_Y, numLayers);
	}
	public int getLayerDown(Coord crd) {
		return layerDown[crd.getX()][crd.getY()];
	}
	public int getLayer(Coord crd) {
		if (crd.getZ() >= numLayers) return 0;
		return layer[crd.getZ()][crd.getX()][crd.getY()];
	}
	public boolean isRampType(Coord crd, int type) {
		//type == 0 rampUp
		//type == 1 ramdRight
		//type == 2 ramdDown
		//type == 3 rampLeft
		int tex = layer[crd.getZ()][crd.getX()][crd.getY()];
		return (tex >= 200) && (tex < 300) && (tex%4 == type);
	}
	
	public boolean isPassable(Coord crd) {
		int tex = layer[crd.getZ()][crd.getX()][crd.getY()];
		return (tex == 0) || ((tex < 200) && (tex >= 100));
	}
	public int getLayerTop(Coord crd) {
		return layerTop[crd.getX()][crd.getY()];
	}
	public boolean isAppleAt(Coord crd) {
		return apples[crd.getZ()][crd.getX()][crd.getY()];
	}
	
	public void setSnake(Coord crd, boolean v) {
		snake[crd.getZ()][crd.getX()][crd.getY()] = v;
	}
	
	public boolean getSnake(Coord crd) {
		return snake[crd.getZ()][crd.getX()][crd.getY()];
	}

	public boolean placeApples(int n, Coord crd) {
		int trys = 0;
		for (int i = 0; i < n ; i++) {
			trys = 0;
			Coord c = Coord.randCoord(Coord.getZeroCoord(), new Coord(Globals.SIZE_X, Globals.SIZE_Y, numLayers));
			if (crd.getX() >= 0) c = crd;
			while ( (!isPassable(c) ||
			         apples[c.getZ()][c.getX()][c.getY()] ||
			         snake[c.getZ()][c.getX()][c.getY()] ||
			         c.equals(crd)) && (trys < 100000)) {
				c = Coord.randCoord(Coord.getZeroCoord(), new Coord(Globals.SIZE_X, Globals.SIZE_Y, numLayers));
				trys++;
			}
			if (trys < 100000) apples[c.getZ()][c.getX()][c.getY()] = true;
			else return false;
		}
		return true;
	}
	public boolean removeApple(Coord c) {
		apples[c.getZ()][c.getX()][c.getY()] = false;
		applesTillNew--;
		numApples++;
		if (applesTillNew == 0) {
			if (type == 2) {
				applesTillNew = applesTillNewGlobal+1;
				applesTillNewGlobal++;
			} else if (type == 1) {
				applesTillNew = 0;
			} else applesTillNew = applesTillNewGlobal;
			return placeApples(applesTillNew, c);
		}
		return true;
	}
}
