/*(c) JippySnake7 Developmentteam.
 *
 *
 * This code is published unter GNU Public Licence v2.
 *
 *
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

class LevelEditor extends JFrame {
	private Canvas cTextures = new Canvas();
	private Canvas cLevel = new Canvas();
	private JPanel pControls = new JPanel();
	
	private JLabel lType = new JLabel("Type:");
	private JLabel lNumber = new JLabel("Number:");
	private JLabel lShownApples = new JLabel("ShownApples:");
	private JLabel lMinApples = new JLabel("MinApples:");
	private JLabel lApplesTillNew = new JLabel("AppleTillNew:");
	private JLabel lLayers = new JLabel("Layers:");
	
	private JTextField tfType = new JTextField();
	private JTextField tfNumber = new JTextField();
	private JTextField tfShownApples = new JTextField();
	private JTextField tfMinApples = new JTextField();
	private JTextField tfApplesTillNew = new JTextField();
	private JTextField tfLayers = new JTextField();
	
	private JButton bLoad = new JButton("Load");
	private JButton bSave = new JButton("Save");
	private JButton bDrawTools = new JButton("drawTools");
	
	private LERenderer lRenderer;
	private LERenderer tRenderer;
	
	private int drawLayer = 0;
	private int[][] textNums;
	private int tex = 0;
	
	private boolean drawOnMove = false;
	
	public LevelEditor() {
		pControls.setLayout(new GridLayout(3,5));
		pControls.add(lType);
		pControls.add(tfType);
		pControls.add(lNumber);
		pControls.add(tfNumber);
		pControls.add(bSave);
		pControls.add(lShownApples);
		pControls.add(tfShownApples);
		pControls.add(lApplesTillNew);
		pControls.add(tfApplesTillNew);
		pControls.add(bLoad);
		pControls.add(lMinApples);
		pControls.add(tfMinApples);
		pControls.add(lLayers);
		pControls.add(tfLayers);
		pControls.add(bDrawTools);
		cTextures.setPreferredSize(new Dimension(450,500));
		cLevel.setPreferredSize(new Dimension(500,500));
		
		setLayout(new BorderLayout());
		add(cTextures, BorderLayout.WEST);
		add(cLevel, BorderLayout.CENTER);
		add(pControls, BorderLayout.SOUTH);
		pack();
		setVisible(true);
		textNums = new int[15][20];
		tRenderer = new LERenderer(cTextures, "res/textures/texturesLE",1, 30);
		lRenderer = new LERenderer(cLevel, "res/textures/texturesLE",7, 25);
		fillTextNums();
		cLevel.requestFocus();

		bDrawTools.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				drawTextures();
			}
		});
		bLoad.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadLevel();
			}
		});
		bSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveLevel();
			}
		});
		cTextures.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				setTexture(e.getX()/30, e.getY()/30);
			}
		});
		cLevel.addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseMoved(MouseEvent e) {
				drawTiles(e.getX()/25, e.getY()/25);
			}
		});
		
		cLevel.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				int c = e.getKeyCode();
				if (c == KeyEvent.VK_0)
					setLayer(0);
				else if (c == KeyEvent.VK_1)
					setLayer(1);
				else if (c == KeyEvent.VK_2)
					setLayer(2);
				else if (c == KeyEvent.VK_3)
					setLayer(3);
				else if (c == KeyEvent.VK_4)
					setLayer(4);
				else if (c == KeyEvent.VK_5)
					setLayer(5);
				else if (c == KeyEvent.VK_6)
					setLayer(6);
				else if (c == KeyEvent.VK_F)
					fillLayer();
				else if (c == KeyEvent.VK_D)
					setDrawTiles(true);
			}
			public void keyReleased(KeyEvent e) {
				int c = e.getKeyCode();
				if (c == KeyEvent.VK_D)
					setDrawTiles(false);
			}
		});
		cLevel.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				draw(new Coord(e.getX()/25,e.getY()/25));
			}
		});
		
	}
	
	private void drawTiles(int x, int y) {
		if (drawOnMove)
			draw(new Coord(x,y));
	}
	private void setDrawTiles(boolean v) {
		drawOnMove = v;
	}
	
	private void draw(Coord c) {
		lRenderer.setTexture(drawLayer, tex, c);
		lRenderer.renderLayer(drawLayer);
		drawLevel();
		
	}
	private void setLayer(int i) {
		drawLayer = i;
		setTitle(""+drawLayer);
	}
	private void fillLayer() {
		for (Coord c : Coord.range(new Coord(0,0,drawLayer), new Coord(19,19,drawLayer)))
			lRenderer.setTexture(drawLayer, tex, c);
		lRenderer.renderLayer(drawLayer);
		drawLevel();
	}
	private void setTexture(int x, int y) {
		tex = textNums[x][y];
		cLevel.requestFocus();
	}
	private void drawTextures() {
		tRenderer.clear(Color.WHITE);
		tRenderer.renderLayer(0);
		tRenderer.drawScreen();
	}
	
	private void drawLevel() {
		lRenderer.clear(Color.GRAY);
		lRenderer.drawScreen();
	}
	
	private void fillTextNums() {
		int n = 0;
		for (Coord c : Coord.range(Coord.getZeroCoord(), new Coord(14,19))) {
			while (tRenderer.getTexture(n).equals("")) n++;
			if (n >= 1000) break;
			textNums[c.getX()][c.getY()] = n;
			n++;
			tRenderer.setTexture(0,textNums[c.getX()][c.getY()],c);
		}
	}
	
	private void loadLevel() {
		int lNum = Integer.parseInt(tfNumber.getText());
		int lType = Integer.parseInt(tfType.getText());
		JSLevel le = JSLevel.loadLevel("res/levels/level"+lType+"_"+lNum+".js7l");
		for (int x = 0; x < 20; x++)
			for(int y = 0; y < 20; y++)
				for(int l = 0; l < 7; l++) {
					Coord c = new Coord(x,y,l-1);
					if (l == 0) lRenderer.setTexture(0,le.getLayerDown(c),c);
					else if (l == 6) lRenderer.setTexture(6,le.getLayerTop(c),c);
					else lRenderer.setTexture(l,le.getLayer(c),c);
				}
		tfShownApples.setText(""+le.getNumShownApples());
		tfMinApples.setText(""+le.getMinApples());
		tfApplesTillNew.setText(""+le.applesTillNew());
		tfLayers.setText(""+le.getNumLayers());
		for (int l = 0; l < 7; l++)
			lRenderer.renderLayer(l);
		drawLevel();
		drawTextures();
	}
	
	private void saveLevel() {
		String tmp;
		try {
			BufferedWriter buffW = new BufferedWriter(new FileWriter("res/levels/level"+tfType.getText()+"_"+tfNumber.getText()+".js7l"));
			//save the level-options
			tmp = "TYPE::"+tfType.getText();
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			tmp = "NUMBER::"+tfNumber.getText();
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			tmp = "LAYERS::"+tfLayers.getText();
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			tmp = "APPLESTOEAT::"+tfMinApples.getText();
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			tmp = "APPLESTILLNEW::"+tfApplesTillNew.getText();
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			tmp = "SHOWNAPPLES::"+tfShownApples.getText();
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			buffW.newLine();
			
			//save down-layer
			tmp = "LAYER_DOWN";
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			
			for (int r = 0; r < 20; r++) {
				tmp = "";
				for (int i = 0; i < 20; i++) {
					tmp = tmp.concat(lRenderer.getTexture(0,new Coord(i,r))+" ");
				}
				buffW.write(tmp,0,tmp.length());
				buffW.newLine();
			}
			buffW.newLine();
			
			//save layers
			for (int n = 0; n < Integer.parseInt(tfLayers.getText()); n++) {
				tmp = "LAYER::"+n;
				buffW.write(tmp,0,tmp.length());
				buffW.newLine();
				
				for (int r = 0; r < 20; r++) {
					tmp = "";
					for (int i = 0; i < 20; i++) {
						tmp = tmp.concat(lRenderer.getTexture(n+1,new Coord(i,r))+" ");
					}
					buffW.write(tmp,0,tmp.length());
					buffW.newLine();
				}
				buffW.newLine();
			}
			
			//save Top-layer
			tmp = "LAYER_TOP";
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			
			for (int r = 0; r < 20; r++) {
				tmp = "";
				for (int i = 0; i < 20; i++) {
					tmp = tmp.concat(lRenderer.getTexture(6,new Coord(i,r))+" ");
				}
				buffW.write(tmp,0,tmp.length());
				buffW.newLine();	
			}
			tmp = "@@@";
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			
			buffW.flush();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}